
package java.lang;

import java.lang.ref.SoftReference;
import java.lang.reflect.*;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.WeakHashMap;
import java.security.AccessControlContext;
import java.security.ProtectionDomain;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.lang.reflect.Proxy;
import sun.misc.Unsafe;
import java.util.concurrent.ThreadPoolExecutor;

import com.ibm.tenant.ParameterizedRunnable;
import com.ibm.tenant.ReturnValue;
import com.ibm.tenant.SlotType;
import com.ibm.tenant.TenantAccess;
import com.ibm.tenant.TenantContext;
import com.ibm.tenant.TenantException;
import com.ibm.tenant.TenantGlobals;
import com.ibm.tenant.internal.TenantAccessFactory;
import com.ibm.jvm.packed.PackedObject;
import com.ibm.jvm.packed.reflect.PackedArray;
import com.ibm.jvm.packed.reflect.PackedField;
import com.ibm.oti.vm.VM;

/*******************************************************************************
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 *     Copyright IBM Corp. 1998, 2014 All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

class J9VMInternals {

	private static final int j9Version = 0x05030270;

	private static final long j9Config = 0x7363617237306200L;	// 'scar70b\0'

	/* Constant filled in by the VM if multitenancy is enabled, equivalent to TenantGlobals.isTenantEnabled() */
	static final boolean MT_ENABLED;

	/* Constant filled in by the VM if tenant debug is enabled, equivalent to TenantGlobals.isDebugEnabled()  */
	private static final boolean MT_DEBUG_ENABLED;

	/* Constant filled in by the VM if tenant JCL support is enabled, equivalent to TenantGlobals.isMTJCLEnabled()  */
	private static final boolean MT_JCL_ENABLED;

	private static ParameterizedRunnable<Void, Object> tenantFinalizeRunnable;

	// cannot create any instances in <clinit> in this special class
	private static Map exceptions;

	private static boolean initialized;
	private static Unsafe unsafe;

	static {
		/* Note this is never called - the VM marks this class as initialized immediately after loading.
		 * The initializer is here solely to trick the compiler into letting us have static final
		 * variables that are initialized by the VM.
		 */
		MT_ENABLED = false;
		MT_DEBUG_ENABLED = false;
		MT_JCL_ENABLED = false;
	}

	/*
	 * Called by the vm after everything else is initialized.
	 */
	private static void completeInitialization() {
		initialized = true;
		exceptions = new WeakHashMap();

		ClassLoader.completeInitialization();
		Thread.currentThread().completeInitialization();
		if (MT_JCL_ENABLED){
			sendTenantCompleteInitialization();

			/* The following two ParameterizedRunnables must be allocated from the ROOT tenant. */
			tenantFinalizeRunnable = new ParameterizedRunnable<Void,Object>() {
				public Void run(Object runnableData) {
					try {
						/* temporary workaround until JAZZ103 46481 is resolved */
						if (runnableData instanceof ThreadPoolExecutor) {
							ThreadPoolExecutor tpe = (ThreadPoolExecutor)runnableData;
							if (tpe.isShutdown()) {
								return null; //do not run finalizer for this object
							}
						}
						runnableData.finalize();
					} catch (java.lang.Throwable e) {}
					return null;
				}
			};

		}
	}

	/**
	 * Check whether -Xaggressive is option is on or not.
	 *
	 * @return true if -Xaggressive is on.
	 * 		   false if -Xaggressive is off.
	 */
	static native boolean isXaggressiveImpl();

	/**
	 * Initialize a Class. See chapter
	 * 2.17.5 of the JVM Specification (2nd ed)
	 */
	static void initialize(Class<?> clazz) {
		/* initializationLock == null means initialization successfully completed */
		if (null != clazz.initializationLock) {
			Unsafe localUnsafe = unsafe;
			if (null == localUnsafe) {
				localUnsafe = unsafe = Unsafe.getUnsafe();
			}
			localUnsafe.ensureClassInitialized(clazz);
		}
	}

	/**
	 * Throw a descriptive NoClassDefFoundError if an attempt is made
	 * to initialize a Class which has previously failed initialization.
	 */
	private static void initializationAlreadyFailed(Class clazz) {
		NoClassDefFoundError notFound = new NoClassDefFoundError(clazz.getName() + " (initialization failure)"); //$NON-NLS-1$
		// if exceptions is null, we're initializing and running single threaded
		if (exceptions != null) {
			synchronized(exceptions) {
				SoftReference weakReason = (SoftReference)exceptions.get(clazz);
				if (weakReason != null) {
					Throwable reason = (Throwable)weakReason.get();
					if (reason != null) {
						reason = copyThrowable(reason);
						notFound.initCause(reason);
					}
				}
			}
		}
		throw notFound;
	}

	/**
	 * Record the cause (Throwable) of a failed initialization for a Class.
	 * If the Throwable is an Error, throw that, otherwise, wrap the Throwable
	 * in an ExceptionInInitializerError and throw that instead.
	 */
	private static void recordInitializationFailure(Class clazz, Throwable err) {
		if (initialized) {
			// if exceptions is null, we're initializing and running single threaded
			if (exceptions == null)
				exceptions = new WeakHashMap();
			synchronized(exceptions) {
				Throwable cause = err;
				if (err instanceof ExceptionInInitializerError) {
					cause = ((ExceptionInInitializerError)err).getException();
				}
				exceptions.put(clazz, new SoftReference(copyThrowable(cause)));
			}
		}
		ensureError(err);
	}

	/**
	 * If the incoming Throwable is an instance of Error, throw it.
	 * Otherwise, wrap it in a new ExceptionInInitializerError and throw that.
	 */
	private static void ensureError(Throwable err) {
		if (err instanceof Error) {
			throw (Error)err;
		}
		throw new ExceptionInInitializerError(err);
	}

	/**
	 * Lifecycle hook called during tenant creation to initialize the class
	 * library.
	 * @param tenant
	 * @throws TenantException
	 */
	static boolean initializeSystem(TenantContext tenant) throws TenantException {
		// Creating a HashMap is the usual way to trigger Oracle class library initialization
		HashMap m = new HashMap();
		System.afterClinitInitialization();
		System.completeInitialization();
		TenantAccess.getAccess().initializeHotClasses(tenant);
		return com.ibm.tenant.internal.TenantAccessFactory.getAccess().completeTenantInitialization(tenant);
	}

	/**
	 * Prepare the receiver for tenant operation.
	 * Allocate slot indices for isolated fields of the receiver.
	 * @param clazz
	 */
	private static void prepareTenant(java.lang.Class clazz) {

		if (MT_DEBUG_ENABLED) {
			VM.tenantDebug(
					"J9VMInternals.prepareTenant(",
					clazz.getName(),
					")");
		}

		final int[] slotCounts = new int[SlotType.TOTAL_SLOT_TYPES];
		getTenantSlotCounts(clazz, slotCounts);

		try {
			ReturnValue<int[][]> retVal = com.ibm.tenant.internal.TenantAccessFactory.getAccess().allocateSlots(slotCounts, clazz);
			associateTenantSlots(retVal.getResult(), clazz);
		} catch (TenantException e) {
			throw new ExceptionInInitializerError(e);
		}
	}

	/**
	 * Check whether the receiver is tenant aware. (Contains isolated fields)
	 *
	 * @param clazz   the receiver
	 * @return true if the receiver is tenant aware.
	 * 		   false if the receiver is just a normal class.
	 */
	static native boolean isTenantAware(Class clazz);

	/**
	 * Get the count of isolated fields(three types) of the receiver.
	 * @param clazz  the receiver
	 * @param an array of counts int[]{32-bit field count, 64-bit field count, object-ref field count}.
	 */
	private static native void getTenantSlotCounts(java.lang.Class clazz, int[] slotCounts);

	/**
	 * Assign the allocated slot indices to fields of the receiver.
	 *
	 * @param slots	 slot indices allocated by TenantDataRepo
	 * @param clazz	 the receiver
	 */
	private static native void associateTenantSlots(int[][] slots, java.lang.Class clazz);

	/**
	 * Get slot indices for isolated fields of the receiver.
	 *
	 * @param clazz  the receiver
	 * @param a double dimension array contains indices of three types of fields.
	 * 		   int[]{int[] 32-bit fields indices, int[] 64-bit fields indices, int[] object-ref fields indices}.
	 */
	private static native void getTenantSlots(java.lang.Class clazz, int[][] slots);

	/**
	 * Call "static void TenantContext.completeInitialization"
	 *
	 * We need to drop to JNI to make this call as completeInitialization has default visibility and is in a different package
	 */
	private static native void sendTenantCompleteInitialization();

	private static native Throwable newInstance(Class exceptionClass, Class constructorClass);

	private static Throwable cloneThrowable(final Throwable throwable, final HashMap hashMapThrowable) {
		return (Throwable)AccessController.doPrivileged(new PrivilegedAction() {
			public Object run() {
				Throwable clone;
				try {
					Class cls = throwable.getClass();
					clone = newInstance(cls, Object.class);
					while (cls != null) {
						Field[] fields = cls.getDeclaredFields();
						for (int i=0; i<fields.length; i++) {
							if (!Modifier.isStatic(fields[i].getModifiers()) &&
									!(cls == Throwable.class && fields[i].getName().equals("walkback")))
							{
								fields[i].setAccessible(true);
								Object value;
								if (cls == Throwable.class && fields[i].getName().equals("cause")) {
									value = clone;
								} else {
									value = fields[i].get(throwable);
									//	Only copy throwable fields whose stacktrace might be kept within Map exceptions
									//	The throwable stored within Map exceptions as WeakReference could be retrieved (before being GCed) later
									if (value instanceof Throwable) {
										value = copyThrowable((Throwable)value, hashMapThrowable);
									}
								}
								fields[i].set(clone, value);
							}
						}
						cls = getSuperclass(cls);
					}
				} catch (Throwable e) {
					clone = new Throwable("Error cloning Throwable (" + e + "). The original exception was: " + throwable.toString());
				}
				return clone;
			}
		});
	}

	/**
	 * Entry method to copy the specified Throwable, invoke copyThrowable(Throwable, HashMap)
	 * to check loop such that we don't go infinite.
	 *
	 * @param throwable the Throwable to copy
	 *
	 * @return a copy of the Throwable
	 */
	private static Throwable copyThrowable(Throwable throwable) {
		HashMap hashMapThrowable = new HashMap();
		return copyThrowable(throwable, hashMapThrowable);
	}

	/**
	 * Copy the specified Throwable, wrapping the stack trace for each
	 * Throwable. Check for loops so we don't go infinite.
	 *
	 * @param throwable the Throwable to copy
	 * @param hashMapThrowable the Throwables already cloned
	 *
	 * @return a copy of the Throwable or itself if it has been cloned already
	 */
	private static Throwable copyThrowable(Throwable throwable, HashMap hashMapThrowable) {
		if (hashMapThrowable.get(throwable) != null) {
			//	stop recursive call here when the throwable has been cloned
			return	throwable;
		}
		hashMapThrowable.put(throwable, throwable);
		Throwable root = cloneThrowable(throwable, hashMapThrowable);
		root.setStackTrace(throwable.getStackTrace());
		Throwable parent = root;
		Throwable cause = throwable.getCause();
		//	looking for causes recursively which will be part of stacktrace	stored into Map exceptions
		while (cause != null && hashMapThrowable.get(cause) == null) {
			hashMapThrowable.put(cause, cause);
			Throwable child = cloneThrowable(cause, hashMapThrowable);
			child.setStackTrace(cause.getStackTrace());
			parent.setCause(child);
			parent = child;
			cause = cause.getCause();
		}
		return root;
	}

	private static void ensureTenantData(Class clazz) {
		int[] slotCounts = new int[SlotType.TOTAL_SLOT_TYPES];
		getTenantSlotCounts(clazz, slotCounts);
		int[][] slots = new int[SlotType.TOTAL_SLOT_TYPES][];
		slots[SlotType.B32] = new int[slotCounts[SlotType.B32]];
		slots[SlotType.B64] = new int[slotCounts[SlotType.B64]];
		slots[SlotType.REF] = new int[slotCounts[SlotType.REF]];
		slots[SlotType.B64_INITSTATUS] = new int[slotCounts[SlotType.B64_INITSTATUS]];
		getTenantSlots(clazz, slots);

		// make sure the slots allocated and initialized for the current tenant
		com.ibm.tenant.internal.TenantAccessFactory.getAccess().ensureAllocationAndInitializeDefaultsForTenant(slots);
	}

	/**
	 * Throw a descriptive NoClassDefFoundError if an attempt is made
	 * to initialize a Class which has previously failed tenant initialization.
	 */
	private static void tenantInitializationAlreadyFailed(Class clazz) {
		throw new NoClassDefFoundError(clazz.getName() + " (tenant initialization failure)"); //$NON-NLS-1$
	}

	/**
	 * Private method to be called by the VM after a Threads dies and throws ThreadDeath
	 * It has to <code>notifyAll()</code> so that <code>join</code> can work properly.
	 * However, it has to be done when the Thread is "thought of" as being dead by other
	 * observer Threads (<code>isAlive()</code> has to return false for the Thread
	 * running this method or <code>join</code> may never return)
	 *
	 * @author		OTI
	 * @version		initial
	 */
	private static void threadCleanup(Thread thread) {
		// don't synchronize the remove! Otherwise deadlock may occur
		try {
			// Leave the ThreadGroup. This is why remove can't be private
			thread.group.remove(thread);
		}
		finally {
			thread.cleanup();

			synchronized(thread) {
				thread.notifyAll();
			}
		}
	}

	private static void checkPackageAccess(final Class clazz, ProtectionDomain pd) {
		final SecurityManager sm = System.getSecurityManager();
		if (sm != null) {
			AccessController.doPrivileged(new PrivilegedAction() {
				public Object run() {
					sm.checkPackageAccess(clazz.getPackageName());
					if (Proxy.isProxyClass(clazz)) {
						ClassLoader	cl = getClassLoader(clazz);
						sun.reflect.misc.ReflectUtil.checkProxyPackageAccess(cl, clazz.getInterfaces());
					}
					return null;
				}
			}, new AccessControlContext(new ProtectionDomain[]{pd}));
		}
	}

	private static void runFinalize(Object obj) {
		if (MT_JCL_ENABLED) {
			TenantContext tenant = TenantContext.getTenantForObject(obj);
			if (null == tenant) {
				try {
					obj.finalize();
				} catch(Throwable e) {}
			} else {
				try {
					TenantAccessFactory.getAccess().primitiveRun(tenant, tenantFinalizeRunnable, obj);
				} catch(Throwable e) {}
			}
		} else {
			try {
				obj.finalize();
			} catch(Throwable e) {}
		}
	}

	static native StackTraceElement[] getStackTrace(Throwable throwable, boolean pruneConstructors);

	private static native void prepareClassImpl(Class clazz);

	/**
	 * Prepare the specified Class. Fill in initial field values, and send
	 * the class prepare event.
	 *
	 * @param clazz the Class to prepare
	 */
	static void prepare(Class clazz) {
		if (clazz.initializationLock == null) {
			/* initializationLock == null means initialization successfully completed */
			return;
		}
		prepareClassImpl(clazz);
	}

	/**
	 * Determines the superclass of specified <code>clazz</code>.
	 * @param clazz The class to introspect (must not be null).
	 * @return The superclass, or null for primitive types and interfaces.
	 */
	static native Class getSuperclass(Class clazz);

	/**
	 * Determines the interfaces implemented by <code>clazz</code>.
	 * @param clazz The class to introspect (must not be null).
	 * @return An array of all interfaces supported by <code>clazz</code>.
	 */
	static native Class[] getInterfaces(Class clazz);

	/**
	 * Determines the ClassLoader for <code>clazz</code>.
	 * @param clazz The class to introspect (must not be null).
	 * @return The ClassLoader which loaded <code>clazz</code>.
	 */
	static native ClassLoader getClassLoader(Class clazz);

	/**
	 * Answers a new instance of the class represented by the
	 * <code>clazz</code>, created by invoking the default (i.e. zero-argument)
	 * constructor. If there is no such constructor, or if the
	 * creation fails (either because of a lack of available memory or
	 * because an exception is thrown by the constructor), an
	 * InstantiationException is thrown. If the default constructor
	 * exists, but is not accessible from the context where this
	 * message is sent, an IllegalAccessException is thrown.
	 *
	 * @param clazz The class to create an instance of.
	 * @return		a new instance of the class represented by the receiver.
	 * @throws		IllegalAccessException if the constructor is not visible to the sender.
	 * @throws		InstantiationException if the instance could not be created.
	 */
	native static Object newInstanceImpl(Class clazz)
		throws IllegalAccessException, InstantiationException;

	/**
	 * Answers an integer hash code for the parameter.
	 * The hash code returned is the same one that would
	 * be returned by java.lang.Object.hashCode(), whether
	 * or not the object's class has overridden hashCode().
	 * Calling with null will cause a crash, so the caller
	 * must check for null.
	 *
	 * @param		anObject	the object
	 * @return		the hash code for the object
	 *
	 * @see			java.lang.Object#hashCode
	 */
	static native int identityHashCode(Object anObject);

	/**
	 * Primitive implementation of Object.clone().
	 * Calling with null will cause a crash, so the caller
	 * must check for null.
	 *
	 * @param		anObject	the object
	 * @return		a shallow copy of anObject
	 *
	 * @see			java.lang.Object#clone
	 */
	static native Object primitiveClone(Object anObject) throws CloneNotSupportedException;

	/**
	 * A class used for synchronizing Class initialization.
	 *
	 * The instance field is currently only for debugging purposes.
	 */
	static class ClassInitializationLock {
		Class theClass;
	}

	/**
	 * Native used to dump a string to the system console for debugging.
	 *
	 * @param 		str String
	 *					the String to display
	 */
	public static native void dumpString(String str);

	/**
	 * This implementation is copied from @ref System.arraycopy().
	 *
	 * This method is called from bootstrap code to avoid loading the
	 * per-tenant isolated System class in a multi-tenant VM before
	 * the class isolation mechanisms are completely initialized
	 */
	static void arraycopy(char[] A1, int offset1, char[] A2, int offset2, int length) {
		if (MT_DEBUG_ENABLED) {
			if (A1 == null || A2 == null) throw new NullPointerException();
			if (offset1 >= 0 && offset2 >= 0 && length >= 0 &&
					length <= A1.length - offset1 && length <= A2.length - offset2)
			{
				// Check if this is a forward or backwards arraycopy
				if (A1 != A2 || offset1 > offset2 || offset1+length <= offset2) {
					for (int i = 0; i < length; ++i) {
						A2[offset2+i] = A1[offset1+i];
					}
				} else {
					for (int i = length-1; i >= 0; --i) {
						A2[offset2+i] = A1[offset1+i];
					}
				}
			} else {
				throw new ArrayIndexOutOfBoundsException();
			}
		} else {
			System.arraycopy(A1, offset1, A2, offset2, length);
		}
	}

	/**
	 * Get the SecurityManager from System.  If the VM has not yet completed
	 * bootstrapping, return null without referencing System in order to avoid
	 * loading System earlier than necessary.
	 *
	 * This is particularly important for MT, where loading System early causes
	 * System to be non-isolated.
	 *
	 * @return		the SecurityManager or null
	 */
	static SecurityManager safeGetSecurityManager() {
		if (initialized) {
			return System.getSecurityManager();
		}
		return null;
	}

}
