package java.lang;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 2012, 2014  All Rights Reserved
 */

import com.ibm.oti.vm.*;

/**
 * Helper class to allow privileged access to classes
 * from outside the java.lang package. Based on sun.misc.SharedSecrets
 * implementation.
 */
class VMAccess implements VMLangAccess {

	private static ClassLoader extClassLoader;

	/**
	 * Set the extension class loader. It can only be set once.
	 *
	 * @param loader the extension class loader
	 */
	static void setExtClassLoader(ClassLoader loader) {
		if (null == extClassLoader) {
			extClassLoader = loader;
		}
	}

	/**
	 * Answer the extension class loader.
	 */
	public ClassLoader getExtClassLoader() {
		return extClassLoader;
	}

	/**
	 * Returns true if parent is the ancestor of child.
	 * Parent and child must not be null.
	 */
	public boolean isAncestor(java.lang.ClassLoader parent, java.lang.ClassLoader child) {
		return parent.isAncestorOf(child);
	}

	/**
	 * Returns the ClassLoader off clazz.
	 */
	public java.lang.ClassLoader getClassloader(java.lang.Class clazz) {
		return J9VMInternals.getClassLoader(clazz);
	}

	/**
	 * Returns the package name for a given class.
	 * clazz must not be null.
	 */
	public java.lang.String getPackageName(java.lang.Class clazz){
		return clazz.getPackageName();
	}

	/**
	 * Returns a BoundMethodHandle that calls {@link Class#cast(Object)} on the passed in class
	 */
	public java.lang.invoke.MethodHandle getClassCastHandle(java.lang.Class clazz) throws IllegalAccessException, NoSuchMethodException {
		return clazz.getClassCastHandle();
	}

	/**
	 * Set the CPU consumption for the current thread to <tt>nanos</tt>.
	 * @param nanos CPU consumption in nanoseconds.
	 */
	public void setThreadCPUTime(long nanos) {
		Thread.currentThread().setCPUTime(nanos);
	}

	/**
	 * Retrieves the saved CPU consumption for the specified thread.
	 * @param thread the thread to query.
	 * @return the saved CPU consumption in nanoseconds.
	 */
	public long getThreadCPUTime(Thread thread) {
		if (null == thread) {
			throw new NullPointerException();
		}
		return thread.getCPUTime();
	}

	/**
	 * Determines if the supplied <tt>clazz</tt> is tenant aware. (Contains isolated fields and/or
	 * requires per-tenant initialization)
	 *
	 * @param clazz   the class to introspect
	 * @return true if the clazz is tenant aware, false otherwise
	 */
	public boolean isTenantAware(Class clazz) {
		if (null == clazz) {
			throw new NullPointerException();
		}
		return J9VMInternals.isTenantAware(clazz);
	}

	/** Invoke the thread's unhandled exception handler. */
	public void uncaughtException(java.lang.Thread thread, Throwable throwable) {
		thread.uncaughtException(throwable);
	}

}
