/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 2011, 2014  All Rights Reserved
 */
package java.lang.invoke;

final class FilterArgumentsHandle extends PassThroughHandle {
	private final MethodHandle   next;
	private final int            startPos;
	private final MethodHandle[] filters;

	protected FilterArgumentsHandle(MethodHandle next, int startPos, MethodHandle[] filters, MethodHandle equivalent) {
		super(equivalent, "FilterArgumentsHandle", infoAffectingThunks(startPos, filters.length, next.type())); //$NON-NLS-1$
		this.next     = next;
		this.startPos = startPos;
		this.filters  = filters;
	}

	FilterArgumentsHandle(FilterArgumentsHandle originalHandle, MethodType newType) {
		super(originalHandle, newType);
		this.next = originalHandle.next;
		this.startPos = originalHandle.startPos;
		this.filters = originalHandle.filters;
	}

	public static FilterArgumentsHandle get(MethodHandle next, int startPos, MethodHandle[] filters, MethodHandle equivalent) {
		return new FilterArgumentsHandle(next, startPos, filters, equivalent);
	}

	// {{{ JIT support

	private static final ThunkTable _thunkTable = new ThunkTable();
	protected final ThunkTable thunkTable(){ return _thunkTable; }

	private static native int numPrefixArgs();
	private static native int numSuffixArgs();
	private static native int numArgsToFilter();
	private static native int filterArguments(MethodHandle[] filters, int filteredArgumentPlaceholder);

	private static Object[] infoAffectingThunks(int startPos, int numFilters, MethodType nextType) {
		// The position, quantity, and target types of filtered arguments affects jitted code.
		// The thunkable signature is sufficient to describe the source types.
		//
		Object[] result = { startPos, numFilters, nextType };
		return result;
	}

	protected final ThunkTuple computeThunks(Object arg) {
		return thunkTable().get(new ThunkKeyWithObjectArray(ThunkKey.computeThunkableType(type()), (Object[])arg));
	}

	@FrameIteratorSkip
	private final int invokeExact_thunkArchetype_X(int argPlaceholder) {
		if (ILGenMacros.isShareableThunk()) {
			undoCustomizationLogic(next);
			undoCustomizationLogic(filters);
		}
		if (!ILGenMacros.isCustomThunk()) {
			doCustomizationLogic();
		}
		return ILGenMacros.invokeExact_X(next, ILGenMacros.placeholder(
			ILGenMacros.firstN(numPrefixArgs(), argPlaceholder),
			filterArguments(filters, ILGenMacros.middleN(numPrefixArgs(), numArgsToFilter(), argPlaceholder)),
			ILGenMacros.lastN(numSuffixArgs(), argPlaceholder)));
	}

	// }}} JIT support

	@Override
	MethodHandle cloneWithNewType(MethodType newType) {
		return new FilterArgumentsHandle(this, newType);
	}

	final void compareWith(MethodHandle right, Comparator c) {
		if (right instanceof FilterArgumentsHandle) {
			((FilterArgumentsHandle)right).compareWithFilterArguments(this, c);
		} else {
			c.fail();
		}
	}

	final void compareWithFilterArguments(FilterArgumentsHandle left, Comparator c) {
		c.compareStructuralParameter(left.startPos, this.startPos);
		c.compareStructuralParameter(left.filters.length, this.filters.length);
		c.compareChildHandle(left.next, this.next);
		for (int i = 0; (i < left.filters.length) && (i < this.filters.length); i++) {
			c.compareChildHandle(left.filters[i], this.filters[i]);
		}
	}
}

