/*******************************************************************************
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 *     Copyright IBM Corp. 2001, 2014 All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package javax.rcm;

import java.util.Properties;

/**
 * This interface describes the attributes of a resource that is managed by the system.
 *
 */
public interface ResourceAttributes {

	/**
	 * Configures the attributes of the resource.
	 * @param properties property list defining how the resource should be configured
	 */
    void configure(Properties properties);
    /**
     * Test if this instance of ResourceAttributes is equal to another.
     * @param obj another instance of ResourceAttributes.
     * @return return true if both of them describe the same resource.
     */
    boolean equals(Object obj);
    /**
     * Gets the granularity used for measuring usage of this resource.
     * @return the used granularity.
     */
    long getGranularity();
    /**
     * Gets the maximum possible elapsed internals since from the last recent measurement moment.
     * @return the max time the usage reported by ResourceDomain.getUsage can be out of data.
     */
    long getMaximumMeasurementDelay();
    /**
     * Gets the name of resource.
     * @return the name of this resource.
     */
    String getName();
    /**
     * Gets the time interval in nanoseconds which is used as rate management by resource domain.
     * @return period for rate management.
     */
    long getRateManagementPeriod();
    /**
     *  Gets the total available quantity of resource
     * @return the total available quantity of resource.
     */
    long getTotalQuantity();
    /**
     * Gets the unit name in which the resource consumption is measured, likes "byte", "milisecond".
     * @return the measurement unit name.
     */
    String getUnitName();

    /**
     * Overrides the hashCode in java.lang.Object.
     * @return the hash code of this instance.
     */
    int hashCode();
    /**
     * Tests if or not the resource is disposable.
     * @return true if the resource is disposable.
     */
    boolean isDisposable();
    /**
     * Tests if or not the resource is reservable.
     * @return true if the resource is reservable.
     */
    boolean isReservable();
    /**
     * Tests if or not the resource is unbounded.
     * @return true  if the resource is unbounded.
     */
    boolean isUnbounded();
    /**
     * Overrides the toString in java.lang.Object.
     * @return he name of this resource.
     */
    String toString();
}
