/*******************************************************************************
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 *     Copyright IBM Corp. 2001, 2014 All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package javax.rcm;

import java.security.Permission;
import java.security.PermissionCollection;
import java.util.Enumeration;
import java.util.Vector;

/**
 * Collects {@link ResourcePermission} objects and allows to query whether a particular
 * permission is implied by it.
 */
final class ResourcePermissionCollection extends PermissionCollection {

    private static final long serialVersionUID = -4338593820495879815L;

    Vector<Permission> permissions = new Vector<Permission>();

    @Override
    public void add(Permission permission) {
        if (isReadOnly()) {
            throw new IllegalStateException();
        }
        if (permission instanceof ResourcePermission) {
            permissions.addElement(permission);
        } else {
            throw new IllegalArgumentException(permission.toString());
        }

    }

    @Override
    public Enumeration<Permission> elements() {
        return permissions.elements();
    }

    @Override
    public boolean implies(Permission permission) {
        if (permission instanceof ResourcePermission) {
            ResourcePermission rp = (ResourcePermission) permission;
            int matchedMask = 0;
            int i = 0;
            while (i < permissions.size()
                    && ((matchedMask & rp.mask) != rp.mask)) {
                // Cast will not fail since we added it
                matchedMask |= ((ResourcePermission) permissions.elementAt(i))
                        .impliesMask(permission);
                i++;
            }
            return ((matchedMask & rp.mask) == rp.mask);
        }
        return false;
    }

}
