/*******************************************************************************
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 *     Copyright IBM Corp. 2001, 2014 All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package javax.rcm;

import java.util.concurrent.Executors;

import com.ibm.tenant.TenantContext;
import com.ibm.tenant.TenantState;
import com.ibm.tenant.internal.TenantAccessFactory;

/**
 * The signal generator for tenant consumer
 */
class TenantCPUSignalGenerator extends AbstractCPUSignalGenerator {

	/**
	 * Constructs a TenantCPUSignalGenerator instance.
	 * @param interval the interval of generating signal.
	 */
	public TenantCPUSignalGenerator(long interval) {
		super(interval);
		com.ibm.tenant.internal.TenantAccessFactory.getAccess().addGlobalListener(CPUThrottlingRunnable.getInstance());
	}

	@Override
	protected void signalCpuThrottling() {
		for(TenantContext tenant : com.ibm.tenant.internal.TenantAccessFactory.getAccess().getAllLive()){
			//filter out root tenant, we don't want to do throttling for threads in root, likes jit or gc.
			if (TenantContext.ROOT_TENANT != tenant ) {
				//throttling tenant in  RUNNING or STOPPING state.
				if(0 != (tenant.getState() & (TenantState.RUNNING|TenantState.STOPPING))) {
					for(Thread t : com.ibm.tenant.internal.TenantAccessFactory.getAccess().getAttachedThreads(tenant)){
						signalAsyncEvent(t, throttlingRunnable);
					}
				}
			}
		}
	}

	@Override
	protected Thread newThread() {
		return TenantAccessFactory.getAccess().getRootWrappedThreadFactory(Executors.defaultThreadFactory()).newThread(this);
	}

}
