/*******************************************************************************
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 *     Copyright IBM Corp. 2001, 2014 All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package javax.rcm.management;

/**
 * Management interface exposed for ResourceAttributes.
 */
public interface ResourceAttributesMBean {
    /**
     * Gets the granularity used for measuring usage of this resource.
     * @return the used granularity.
     */
	long getGranularity();
    /**
     * Gets the maximum possible elapsed internals since from the last recent measurement moment.
     * @return the max time the usage reported by ResourceDomain.getUsage can be out of data.
     */
    long getMaximumMeasurementDelay();
    /**
     * Gets the time interval in nanoseconds which is used as rate management by resource domain.
     * @return period for rate management.
     */
    long getRateManagementPeriod();
    /**
     *  Gets the total available quantity of resource
     * @return the total available quantity of resource.
     */
    long getTotalQuantity();
    /**
     * Gets the unit name in which the resource consumption is measured, likes "byte", "milisecond".
     * @return the measurement unit name.
     */
    String getUnitName();
    /**
     * Tests if or not the resource is disposable.
     * @return true if the resource is disposable.
     */
    boolean isDisposable();
    /**
     * Tests if or not the resource is reservable.
     * @return true if the resource is reservable.
     */
    boolean isReservable();
    /**
     * Tests if or not the resource is unbounded.
     * @return true  if the resource is unbounded.
     */
    boolean isUnbounded();
}
