/*******************************************************************************
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 *     Copyright IBM Corp. 2001, 2014 All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package javax.rcm.management;

/**
 * Management interface exposed for ResourceDomain.
 */
public interface ResourceDomainMBean {
    /**
     * Gets the reservation associated with this instance.
     * @return the reservation associated with this resource domain.
     */
    public long getReservation();
    /**
     * Gets the resource name controlled by this domain.
     * @return the managed resource name.
     */
    public String getResourceAttributesName();
    /**
     * Get the total usage of resource in all resource domains.
     * @return the total usage of resource.
     */
    public long getTotalQuantity();
    /**
     * Gets the total reserved amount of the resource which is controlled by this domain instnace.
     * @return the total reserved amount of the resource.
     */
    public long getTotalReservation();
    /**
     * Get the total usage of resource in all resource domains.
     * @return the total usage of resource.
     */
    public long getTotalUsage();
    /**
     * Gets the total rate of usage of the resource in all resource domains.
     * @return the total rate of usage of the resource.
     */
    public long getTotalUsageRate();
    /**
     * Gets the usage of the resource in this resource domain.
     * @return the usage of the resource.
     */
    public long getUsage();
    /**
     * Gets the total rate of usage of the resource in this resource domain.
     * @return the rate of usage of the resource.
     */
    public long getUsageRate();
    /**
     * Tests if the domain is in the terminated state.
     * @return true it is in terminated state.
     */
    public boolean isTerminated();
}
